# ***************************************************************************
# Copyright 2019 VMware, Inc.  All rights reserved. VMware Confidential.
# ***************************************************************************
# $Id$
# $DateTime$
# $Change$
# $Author$
# ***************************************************************************

from cli.commands.cli_command_provider import CliCommandProvider
from cli.utils.constants import NODE_TYPE_EDGE
from cli.utils.constants import NODE_TYPE_AUTONOMOUS_EDGE
from cli.utils.constants import NODE_TYPE_HOST_CENTOSKVM
from cli.utils.constants import NODE_TYPE_HOST_SLESKVM
from cli.utils.constants import NODE_TYPE_HOST_ESX
from cli.utils.constants import NODE_TYPE_MANAGER
from cli.utils.constants import NODE_TYPE_CONTROLLER
from cli.utils.constants import NODE_TYPE_HOST_RHELKVM
from cli.utils.constants import NODE_TYPE_HOST_UBUNTUKVM
from cli.utils.constants import NODE_TYPE_BAREMETAL_UBUNTU_SERVER
from cli.utils.constants import NODE_TYPE_BAREMETAL_RHEL_SERVER
from cli.utils.constants import NODE_TYPE_BAREMETAL_CENTOS_SERVER
from cli.utils.constants import NODE_TYPE_BAREMETAL_SLES_SERVER
from cli.utils.constants import NODE_TYPE_BAREMETAL_RHEL_CONTAINER
from cli.utils.constants import NODE_TYPE_BAREMETAL_CENTOS_CONTAINER
from cli.utils.error_code import ErrorCode as ec
from cli.utils.node_utils import get_node_type
from cli.utils.signal_utils import send_sighup

import errno
import json
from mako import compat
import nsx_logging
import os

SERVICE_NAME = "nsx-platform-client"

lg = nsx_logging.getLogger("cli.commands.common.platform_client_log_level")

# Use nsx-manager if exists, else use the first node type
node_types = get_node_type().split()
if NODE_TYPE_MANAGER in node_types:
    g_node_type = NODE_TYPE_MANAGER
else:
    g_node_type = node_types[0]

LOGGING_LEVEL_MAP = {
    "off": "0",
    "fatal": "1",
    "error": "2",
    "warn": "3",
    "info": "4",
    "debug": "5",
    "trace": "6"
}


# ----------------------------------------------------------------------
# set platform client service logging level
# ----------------------------------------------------------------------
class SetPlatformClientLoggingLevel(CliCommandProvider):
    def __call__(self, cmd_list):
        level = self.get_arg_value("platform-client-logging-level-arg", cmd_list)
        if self.set_logging_level(SERVICE_NAME, level) == True:
            return "OK\n\n%s" % json.dumps("")
        else:
            return "MSG_SET_LOG_LEVEL_FAILED"

    def update_config_file(self, daemon_name, level):
        #  Updates daemon's config file with specified log level
        dirpath = "/etc/vmware/"+ daemon_name
        filename = dirpath + "/config.json"
        logLevel = LOGGING_LEVEL_MAP[level]
        if os.path.isfile(filename):
            with open(filename, 'r+') as f:
                data = json.load(f)
                data['logLevel'] = logLevel
                f.seek(0)
                json.dump(data, f, indent=4)
        else:
            if g_node_type in [NODE_TYPE_EDGE, NODE_TYPE_AUTONOMOUS_EDGE, NODE_TYPE_CONTROLLER, NODE_TYPE_MANAGER]:
                lg.error("Config file %s not found", filename, error_code=ec.OPERATION_FAILED)
                return False

            try:
                original_umask = os.umask(0)
                # if nsx-platform-client directory does not exist then create it
                if not os.path.isdir(dirpath):
                    os.makedirs(os.path.dirname(filename), compat.octal("0775"))
            except OSError as exc:
                if exc.errno != errno.EEXIST:
                    lg.error("Could not create config dir %s exception %s",
                                os.path.dirname(filename), exc, error_code=ec.OPERATION_FAILED)
                    return False
            finally:
                os.umask(original_umask)
            with open(filename, 'w') as f:
                data = {}
                data["logLevel"] = logLevel
                json.dump(data, f, indent=4)

        return True

    def set_logging_level(self, daemon_name, level):
        #  Updates daemon's config file with specified log level. Then sends
        #  SIGHUP signal to the daemon.
        #  Returns true if successful, otherswise returns false.
        if not self.update_config_file(daemon_name, level):
            return False

        if send_sighup(daemon_name):
            return True
        else:
            lg.error("Failed to send SIGHUP to service %s", daemon_name, error_code=ec.OPERATION_FAILED)
            return False
